
import { createHash } from 'crypto';
import { random_bytes } from './random-bytes';
import { invalidate_cookie, parse_req_cookies, set_cookie } from './cookies';

export interface PKCECookieConfig {
	/**
	 * The name of the cookie to store the PKCE code challenge in
	 */
	name: string;

	/**
	 * Should the cookie only be sent over secure connections?
	 */
	secure: boolean;

	/**
	 * Time-to-live (in seconds) for the PKCE code challenge cookie
	 */
	ttl: number;

	/**
	 * Number of bytes of random data to generate for the verification
	 * code (more is stronger, must be in range 32-96)
	 */
	code_bytes: number;
}

export type PKCECookieProvider = ReturnType<typeof create_pkce_cookie_provider>;

export function create_pkce_cookie_provider(conf: PKCECookieConfig) {
	return Object.freeze({
		async prepare_pkce_challenge() {
			const pkce_code_verifier = await random_bytes(conf.code_bytes, 'base64url');
			const pkce_code_challenge = sha256_base64_url(pkce_code_verifier);
			const pkce_expire = new Date(Date.now() + (conf.ttl * 1000));
			
			// "Lax" rather than "Strict", so the PKCE verifier will be included on the
			// redirect to the login callback endpoint, which comes from the OpenID server,
			// which is likely on a different site
			const set_cookie_header = set_cookie(conf.name, pkce_code_verifier, pkce_expire, conf.secure, 'Lax');
			
			return {
				set_cookie_header,
				pkce_code_challenge,
			};
		},

		read_pkce_code_verifier(cookies_header: string) {
			const cookies = parse_req_cookies(cookies_header);
			return cookies[conf.name];
		},

		invalidate_cookie: invalidate_cookie(conf.name, conf.secure),
	});
}

function sha256_base64_url(input: string) {
	const hash = createHash('sha256');
	hash.update(input);
	return hash.digest('base64url');
}
