
import { SessionToken, generate_session_token, parse_session_token } from './session-token';
import { invalidate_cookie, parse_req_cookies, set_cookie } from './cookies';

export interface SesionCookieConfig {
	/**
	 * The name of the cookie to store the session token in
	 */
	name: string;

	/**
	 * Should the cookie only be sent over secure connections?
	 */
	secure: boolean;

	/**
	 * Time-to-live (in seconds) for the session token cookie
	 */
	ttl: number;

	/**
	 * Cryptographic hashing function to use to protect token secrets
	 * while stored. Good choices would be argon2, bcrypt, scrypt
	 */
	hash(secret: string) : string | Promise<string>;

	/**
	 * Verification function for the provided hashing function
	 */
	verify_hash(secret: string, hashed_secret: string) : boolean | Promise<boolean>;
}

export interface ReadSessionCredentials {
	bearer?: SessionToken;
	cookie?: SessionToken;
}

export interface SessionTokenWithSource extends SessionToken {
	from: 'bearer' | 'cookie';
}

export type SessionCredentialProvider = ReturnType<typeof create_session_credentials_provider>;

export function create_session_credentials_provider(conf: SesionCookieConfig) {
	return Object.freeze({
		async prepare_new_session_credentials() {
			const session_token = await generate_session_token();
			const hashed_secret = await conf.hash(session_token.secret);
			const session_expire = new Date(Date.now() + (conf.ttl * 1000));
			const set_cookie_header = set_cookie(conf.name, session_token.token, session_expire, conf.secure, 'Strict');
			
			return {
				set_cookie_header,
				session_token,
				hashed_secret,
			};
		},

		read_session_credentials(authorization_header: string, cookies_header: string) : SessionTokenWithSource {
			if (authorization_header?.startsWith('Bearer ')) {
				const token = parse_session_token(authorization_header.slice(7));
				return Object.assign(token, { from: 'bearer' as const });
			}

			const cookies = parse_req_cookies(cookies_header);

			if (cookies[conf.name]) {
				const token = parse_session_token(cookies[conf.name]);
				return Object.assign(token, { from: 'cookie' as const });
			}
		},

		invalidate_cookie: invalidate_cookie(conf.name, conf.secure),

		async verify_session_credentials(token: SessionToken, hashed_secret: string) : Promise<boolean> {
			return conf.verify_hash(token.secret, hashed_secret);
		},
	});
}
