
import { random_bytes } from './random-bytes';

/**
 * 
 */
export interface SessionToken {
	/**
	 * The non-secret segment of the token; This should be used as an
	 * identifier for the session, and as such will likely be stored
	 * in plain-text as part of an index. This is a 12-byte, base64url
	 * encoded string (16 characters encoded)
	 */
	prefix: string;

	/**
	 * The secret segment of the token; This should be hashed and stored
	 * for later validation, similar to how you would treat a password. This
	 * is a 36-byte, base64url encoded string (48 characters encoded)
	 */
	secret: string;

	/**
	 * The full token value to be provided to the client in a cookie; A
	 * concatenation of the prefix and key segments. This string is 65
	 * character long string (16 char prefix + 48 char secret + 1 char
	 * period ["."] delimiter)
	 */
	token: string;
}

export async function generate_session_token() : Promise<SessionToken> {
	const bytes = await random_bytes(48);
	const base64 = bytes.toString('base64url');
	const prefix = base64.slice(0, 16);
	const secret = base64.slice(16);
	const token = `${prefix}.${secret}`;
	
	return { prefix, secret, token };
}

export function parse_session_token(token: string) : SessionToken {
	const [ prefix, secret, ...rest ] = token.split('.');
		
	if (rest && rest.length || prefix?.length !== 16 || secret?.length !== 48) {
		throw new Error('invalid session token');
	}

	return { prefix, secret, token };
}
